@@ -1,4 +1,4 @@
-# $Id: summary.genotype.R 448 2005-11-08 20:54:14Z warnes $
+# $Id: summary.genotype.R 1280 2007-07-25 07:32:38Z ggorjan $
 
 ###
 ### Provide the frequency and proportions of alleles and genotypes
@@ -21,16 +21,16 @@ summary.genotype  <-  function(object,...,maxsum)
     # summary.factor so that we don't mess up the display
     if(!missing(maxsum))
       return(shortsummary.genotype(object,...,maxsum=maxsum))
-    
+
     retval  <-  list()
 #    retval$genotype  <- object
     retval$allele.names  <- allele.names(object)
-    
+
     retval$locus  <- attr(object,"locus")
     class(retval)  <- "summary.genotype"
 
     af  <- table(allele(object))
-    
+
     # make sure af has same order as allele.names...
     #
     missed <- !names(af) %in% retval$allele.names 
@@ -38,7 +38,7 @@ summary.genotype  <-  function(object,...,maxsum)
     names(af.tab) <- retval$allele.names
     af.tab[names(af)] <- af
     #
-    
+
     paf <- prop.table(af.tab)
     retval$allele.freq    <- cbind("Count"=af.tab,"Proportion"=paf)
 
@@ -46,6 +46,13 @@ summary.genotype  <-  function(object,...,maxsum)
     pgf <- prop.table(gf)
     retval$genotype.freq  <- cbind("Count"=gf,"Proportion"=pgf)
 
+    ## Sort by genotypeOrder
+    asFun <- as.genotype
+    if(is.haplotype(object)) asFun <- as.haplotype
+    tmp <- asFun(rownames(retval$genotype.freq),
+                 alleles=allele.names(object))
+    tmp <- sort(tmp, genotypeOrder=genotypeOrder(object))
+    retval$genotype.freq <- retval$genotype.freq[tmp, ]
 
     ### from code submitted by David Duffy <davidD@qimr.edu.au>
     #
@@ -64,7 +71,7 @@ summary.genotype  <-  function(object,...,maxsum)
     #
     ###
 
-    
+    ## Add info on NA values
     if(any(is.na(object))){
         retval$allele.freq    <- rbind(retval$allele.freq,
                                        "NA"=c(sum(is.na(allele(object))),NA))
@@ -72,9 +79,6 @@ summary.genotype  <-  function(object,...,maxsum)
                                        "NA"=c(sum(is.na(object)),NA))
     }
 
-
-
-    
     return(retval)
   }
 
@@ -100,11 +104,10 @@ print.summary.genotype  <-  function(x,...,round=2)
     cat("Genotype Frequency:\n")
     print(round(x$genotype.freq,digits=round),...)
     cat("\n")
-    
+
     cat("Heterozygosity (Hu)  = ",  x$Hu, "\n", sep="")
     cat("Poly. Inf. Content   = ",  x$pic, "\n", sep="")
     cat("\n")
-    
+
     invisible(x)
   }
-
